@@ -87,11 +87,15 @@ def emscript(infile, settings, outfile, libraries=None, compiler_engine=None,
     # these functions are split up to force variables to go out of scope and allow
     # memory to be reclaimed
 
-    funcs, metadata, mem_init = get_and_parse_backend(infile, settings, temp_files, DEBUG)
-    glue, forwarded_data = compiler_glue(metadata, settings, libraries, compiler_engine, temp_files, DEBUG)
+    with ToolchainProfiler.profile_block('get_and_parse_backend'):
+      funcs, metadata, mem_init = get_and_parse_backend(infile, settings, temp_files, DEBUG)
+    with ToolchainProfiler.profile_block('compiler_glue'):
+      glue, forwarded_data = compiler_glue(metadata, settings, libraries, compiler_engine, temp_files, DEBUG)
 
-    post, funcs_js, need_asyncify, provide_fround, asm_safe_heap, sending, receiving, asm_setup, the_global, asm_global_vars, asm_global_funcs, pre_tables, final_function_tables, exports, last_forwarded_json = function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data, settings, outfile, DEBUG)
-    finalize_output(metadata, post, funcs_js, need_asyncify, provide_fround, asm_safe_heap, sending, receiving, asm_setup, the_global, asm_global_vars, asm_global_funcs, pre_tables, final_function_tables, exports, last_forwarded_json, settings, outfile, DEBUG)
+    with ToolchainProfiler.profile_block('function_tables_and_exports'):
+      post, funcs_js, need_asyncify, provide_fround, asm_safe_heap, sending, receiving, asm_setup, the_global, asm_global_vars, asm_global_funcs, pre_tables, final_function_tables, exports, last_forwarded_json = function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data, settings, outfile, DEBUG)
+    with ToolchainProfiler.profile_block('finalize_output'):
+      finalize_output(metadata, post, funcs_js, need_asyncify, provide_fround, asm_safe_heap, sending, receiving, asm_setup, the_global, asm_global_vars, asm_global_funcs, pre_tables, final_function_tables, exports, last_forwarded_json, settings, outfile, DEBUG)
 
     success = True
 
@@ -136,13 +140,16 @@ def get_and_parse_backend(infile, settings, temp_files, DEBUG):
         backend_args += ['-emscripten-no-exit-runtime']
       if settings['BINARYEN']:
         backend_args += ['-emscripten-wasm']
+        if shared.Building.is_wasm_only():
+          backend_args += ['-emscripten-only-wasm']
       if settings['CYBERDWARF']:
         backend_args += ['-enable-cyberdwarf']
 
       if DEBUG:
         logging.debug('emscript: llvm backend: ' + ' '.join(backend_args))
         t = time.time()
-      shared.jsrun.timeout_run(subprocess.Popen(backend_args, stdout=subprocess.PIPE))
+      with ToolchainProfiler.profile_block('emscript_llvm_backend'):
+        shared.jsrun.timeout_run(subprocess.Popen(backend_args, stdout=subprocess.PIPE))
       if DEBUG:
         logging.debug('  emscript: llvm backend took %s seconds' % (time.time() - t))
 
@@ -689,6 +696,9 @@ function _emscripten_asm_const_%s(%s) {
       basic_funcs += ['setTempRet0', 'getTempRet0']
       asm_setup += 'var setTempRet0 = Runtime.setTempRet0, getTempRet0 = Runtime.getTempRet0;\n'
 
+    if settings['BINARYEN']:
+      asm_setup += "\nModule['wasmTableSize'] = %d;\n" % sum(map(lambda table: table.count(',') + 1, last_forwarded_json['Functions']['tables'].values()))
+
     # See if we need ASYNCIFY functions
     # We might not need them even if ASYNCIFY is enabled
     need_asyncify = '_emscripten_alloc_async_context' in exported_implemented_functions
@@ -1559,8 +1569,8 @@ return ASM_CONSTS[code](%s);
   with open(wasm) as f:
     for line in f:
       if line.startswith('  (import '):
-        parts = line.split(' ')
-        func_name = parts[3][1:]
+        parts = line.split()
+        func_name = parts[2][1:-1]
         if func_name.startswith('invoke_'):
           sig = func_name[len('invoke_'):]
           invoke_wrappers += '\n' + shared.JS.make_invoke(sig) + '\n'
